<?php
/**
 *  Adyen Payment Module
 *  osCommerce, Open Source E-Commerce Solutions
 *  http://www.oscommerce.com
 *  Copyright (c) 2007 osCommerce
 *
 *  Released under the GNU General Public License
 *
 */


class adyen {
  var $code, $title, $description, $enabled;

  function adyen() {
	$this->code 			= 'adyen';
	$this->title			= MODULE_PAYMENT_ADYEN_TEXT_TITLE;
	$this->description	 	= MODULE_PAYMENT_ADYEN_TEXT_DESCRIPTION;
	$this->sort_order 		= MODULE_PAYMENT_ADYEN_SORT_ORDER;
	$this->enabled		 	= MODULE_PAYMENT_ADYEN_STATUS;
	
	if($this->isOnePagePayment()) {
		$landing_page = "pay.shtml";
	} else {
		$landing_page = "select.shtml";
	}
	
	if($this->isTestMode()) {
		$this->form_action_url 	= 'https://test.adyen.com/hpp/' . $landing_page;
	} else {
		$this->form_action_url 	= 'https://live.adyen.com/hpp/' . $landing_page;
	}
  }

  function javascript_validation() {
	return true;
  }

  function selection() {
	return array('id' => $this->code, 'module' => $this->title);
  }

  function pre_confirmation_check() {
	return false;
  }

  function confirmation() {
	return false;
  }

  function process_button() {
	global $customer_id, $order, $currencies;

	$adyen_orderID = $customer_id . 'AT' . date('YmdHis');
	$adyen_skinCode = MODULE_PAYMENT_ADYEN_SKIN;
	$adyen_merchantAccount = MODULE_PAYMENT_ADYEN_MERCHANTACCOUNT;
	$adyen_amount = number_format($order->info['total'] * 100 * $order->info['currency_value'], 0, '', '');
	$adyen_currency = $order->info['currency'];
	$adyen_shipbefore = date('Y-m-d',  mktime(0, 0, 0, date("m")  , date("d")+MODULE_PAYMENT_ADYEN_SHIPBEFORE, date("Y")));
	$adyen_countryCode = MODULE_PAYMENT_ADYEN_COUNTRY;
	$adyen_shopperEmail = $order->customer['email_address'];
	$adyen_shopperReference = $customer_id;
	
	// for PHP4 and PHP5 < 5.13, DATE_ATOM is not defined
	if(DATE_ATOM == "DATE_ATOM") {	
		$dateformat = "Y-m-d\TH:i:s\Z";
	} else {
		$dateformat = DATE_ATOM;
	}
	
	$adyen_sessionValidity =  gmdate($dateformat, time() + MODULE_PAYMENT_ADYEN_SESSIONVALID);
	
	$adyen_locale = MODULE_PAYMENT_ADYEN_LANGUAGE;

	$process_button_string = "\n" .
							 tep_draw_hidden_field('merchantReference', $adyen_orderID) . "\n" .
							 tep_draw_hidden_field('paymentAmount', $adyen_amount) . "\n" .
							 tep_draw_hidden_field('currencyCode', $adyen_currency) . "\n" .
							 tep_draw_hidden_field('shipBeforeDate', $adyen_shipbefore) . "\n" .
							 tep_draw_hidden_field('sessionValidity', $adyen_sessionValidity) . "\n" .
							 tep_draw_hidden_field('skinCode', $adyen_skinCode) . "\n" .
							 tep_draw_hidden_field('merchantAccount', $adyen_merchantAccount) . "\n" .
							 tep_draw_hidden_field('shopperLocale', $adyen_locale) . "\n" .
							 tep_draw_hidden_field('shopperEmail', $adyen_shopperEmail) . "\n" .
							 tep_draw_hidden_field('shopperReference', $adyen_shopperReference);

	$sign = $adyen_amount . $adyen_currency . $adyen_shipbefore .  $adyen_orderID . $adyen_skinCode .  $adyen_merchantAccount 
			. $adyen_sessionValidity . $adyen_shopperEmail . $adyen_shopperReference;
 	
	if($this->isTestMode()) {
		$process_button_string .= tep_draw_hidden_field('merchantSig', base64_encode(pack('H*',$this->hmacsha1(MODULE_PAYMENT_ADYEN_HMAC_STRING_TEST,$sign))) ) . "\n";
	} else {
		$process_button_string .= tep_draw_hidden_field('merchantSig', base64_encode(pack('H*',$this->hmacsha1(MODULE_PAYMENT_ADYEN_HMAC_STRING_LIVE,$sign))) ) . "\n";
	}
	
	if($adyen_countryCode != "--") {
		$process_button_string .= tep_draw_hidden_field('countryCode', $adyen_countryCode) . "\n";
	}
	
	return $process_button_string;
  }

  function before_process() {
    global $order;
    
    $authResult = $_GET['authResult'];
    $pspReference = $_GET['pspReference'];
    $merchantReference  = $_GET['merchantReference'];
    $skinCode  = $_GET['skinCode'];
    $merchantSig  = $_GET['merchantSig'];
    
    $verifiedHash = false;
    
    $responseArray = array();
    $responseArray = $_GET;
    
    // Send the test information to the notify email
	if($this->isTestMode()) $this->sendNotifyEmail("Test Transaction", $responseArray);
    
    // No signature -> no payment
    if(!empty($merchantSig)) {  
      $sign = $authResult . $pspReference . $merchantReference . $skinCode;
      if($this->isTestMode()) {
	  	$computedSig = base64_encode(pack('H*',$this->hmacsha1(MODULE_PAYMENT_ADYEN_HMAC_STRING_TEST,$sign)));
	  } else {
	  	$computedSig = base64_encode(pack('H*',$this->hmacsha1(MODULE_PAYMENT_ADYEN_HMAC_STRING_LIVE,$sign)));
	  }
	  if($computedSig == $merchantSig) {
	  	$verifiedHash = true;
	  }
    }
    
    if($verifiedHash == false) {
    	tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error='.$this->code.'&error='.urlencode("HMAC_INVALID"), 'SSL', true, false));
    }
    
    if($authResult == "REFUSED") {
    	tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error='.$this->code.'&error='.urlencode("REFUSED"), 'SSL', true, false));
    };
    
    if($authResult == "PENDING") {
    	tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error='.$this->code.'&error='.urlencode("PENDING"), 'SSL', true, false));
    };
    
    if($authResult == "AUTHORISED") {
    	$order->info['cc_number'] = $pspReference;
    	$this->sendNotifyEmail("Authorised", $responseArray);
    } else {
    	tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error='.$this->code.'&error='.urlencode("ERROR"), 'SSL', true, false));
    }
  }

  function after_process() {
	return false;
  }

  function output_error() {
	return false;
  }

  function check() {
	if (!isset($this->check)) {
	  $check_query = tep_db_query("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'MODULE_PAYMENT_ADYEN_STATUS'");
	  $this->check = tep_db_num_rows($check_query);
	}
	return $this->check;
  }

  function install() {

	tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Allow ADYEN Payments', 'MODULE_PAYMENT_ADYEN_STATUS', 'True', 'Do you want to accept ADYEN payments?', '6', '20', 'tep_cfg_select_option(array(\'True\', \'False\'), ', now())");
	tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('ADYEN Status Mode', 'MODULE_PAYMENT_ADYEN_MODE', 'Test', 'Status mode for ADYEN payments?', '6', '21', 'tep_cfg_select_option(array(\'Test\', \'Live\'), ', now())");
	tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Payment Flow', 'MODULE_PAYMENT_ADYEN_ONEPAGE', 'Multiple Pages', 'Should the payment process be split over multiple pages?', '6', '22', 'tep_cfg_select_option(array(\'Multiple Pages\', \'Single Page\'), ', now())");
	tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('ADYEN Merchant Account', 'MODULE_PAYMENT_ADYEN_MERCHANTACCOUNT', 'YourAccountHere', 'Merchant Account', '6', '23', now())");
	tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Days Till Delivery', 'MODULE_PAYMENT_ADYEN_SHIPBEFORE', '7', 'Number of days before items are shipped', '6', '27', now())");
	tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Session Valid Till', 'MODULE_PAYMENT_ADYEN_SESSIONVALID', '1800', 'Number of seconds before which the payment must be completed', '6', '28', now())");
	tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('ADYEN Client Language', 'MODULE_PAYMENT_ADYEN_LANGUAGE', 'en_GB', 'Client language', '6', '26', 'tep_cfg_pull_down_adyen_language(', now())");
	tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('ADYEN Client Country', 'MODULE_PAYMENT_ADYEN_COUNTRY', '--', 'Client country used for payment method selection', '6', '27', 'tep_cfg_pull_down_adyen_country(', now())");
	tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('ADYEN HMAC Secret Test String', 'MODULE_PAYMENT_ADYEN_HMAC_STRING_TEST', 'YourHMACSecretTest', 'HMAC string used for the signature on the test system', '6', '28', now())");
	tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('ADYEN HMAC Secret Live String', 'MODULE_PAYMENT_ADYEN_HMAC_STRING_LIVE', 'YourHMACSecretLive', 'HMAC string used for the signature on the live system', '6', '29', now())");
	tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('ADYEN skin Code', 'MODULE_PAYMENT_ADYEN_SKIN', 'YourSkin', 'Adyen Skin Code', '6', '30', now())");
	tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Status Email', 'MODULE_PAYMENT_ADYEN_EMAIL', '', 'The email address of the person responsible for payments in your organisation', '6', '31', now())");
	tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Sort order of display.', 'MODULE_PAYMENT_ADYEN_SORT_ORDER', '0', 'Sort order of display. Lowest is displayed first.', '6', '32', now())");
  }

  function remove() {
	tep_db_query("delete from " . TABLE_CONFIGURATION . " where configuration_key in ('" . join($this->keys(), "','") . "')");
  }

	function keys() {
	return array(
	  'MODULE_PAYMENT_ADYEN_STATUS',
	  'MODULE_PAYMENT_ADYEN_MODE',
	  'MODULE_PAYMENT_ADYEN_ONEPAGE',
	  'MODULE_PAYMENT_ADYEN_MERCHANTACCOUNT',
	  'MODULE_PAYMENT_ADYEN_SHIPBEFORE',
	  'MODULE_PAYMENT_ADYEN_SESSIONVALID',
	  'MODULE_PAYMENT_ADYEN_LANGUAGE',
	  'MODULE_PAYMENT_ADYEN_COUNTRY',
	  'MODULE_PAYMENT_ADYEN_HMAC_STRING_TEST',
	  'MODULE_PAYMENT_ADYEN_HMAC_STRING_LIVE',
	  'MODULE_PAYMENT_ADYEN_SKIN',
	  'MODULE_PAYMENT_ADYEN_EMAIL',
	  'MODULE_PAYMENT_ADYEN_SORT_ORDER'
	);
	}

	/**
	 * Create an array that represents the possible module errors
	 * @return Array Errors that have arrived from HTTP
	 */
	function get_error() {
		global $HTTP_GET_VARS, $language;

		// ensure we have all texts defined
		require_once(DIR_WS_LANGUAGES . $language . '/modules/payment/'.$this->code.'.php');

		$error = '';
		
		if(isset($HTTP_GET_VARS['error'])) {
			$error = urldecode($HTTP_GET_VARS['error']); // otherwise default error is displayed	
		}
		
		switch($error) {
			case 'REFUSED':
				$error_text['title'] = MODULE_PAYMENT_ADYEN_TEXT_REFUSED;
				$error_text['error'] = MODULE_PAYMENT_ADYEN_TEXT_REFUSED_DESCRIPTION;
				break;
			case 'HMAC_INVALID':
				$error_text['title'] = MODULE_PAYMENT_ADYEN_TEXT_HMAC_INVALID;
				$error_text['error'] = MODULE_PAYMENT_ADYEN_TEXT_HMAC_INVALID_DESCRIPTION;
				break;
			default: //other error
				$error_text['title'] = MODULE_PAYMENT_ADYEN_TEXT_ERROR;
				$error_text['error'] = MODULE_PAYMENT_ADYEN_TEXT_ERROR_DESCRIPTION;
				break;
	    	}
	   	return $error_text;
	}


	//Calculate HMAC-SHA1 according to RFC2104
	// http://www.ietf.org/rfc/rfc2104.txt
	function hmacsha1($key,$data) {
	    $blocksize=64;
	    $hashfunc='sha1';
	    if (strlen($key)>$blocksize)
	        $key=pack('H*', $hashfunc($key));
	    $key=str_pad($key,$blocksize,chr(0x00));
	    $ipad=str_repeat(chr(0x36),$blocksize);
	    $opad=str_repeat(chr(0x5c),$blocksize);
	    $hmac = pack(
	                'H*',$hashfunc(
	                    ($key^$opad).pack(
	                        'H*',$hashfunc(
	                            ($key^$ipad).$data
	                        )
	                    )
	                )
	            );
	    return bin2hex($hmac);
	}

	/**
	 * Returns with the Email to Notify of Transaction Errors
	 * @return String Email address of Account Admin
	 */
	function getNotifyEmailAddress() {
		if(defined('MODULE_PAYMENT_ADYEN_EMAIL') && tep_validate_email(MODULE_PAYMENT_ADYEN_EMAIL)) {
			return MODULE_PAYMENT_ADYEN_EMAIL;
		} else {
			if(defined('STORE_OWNER_EMAIL_ADDRESS') && tep_validate_email(STORE_OWNER_EMAIL_ADDRESS)) {
				return STORE_OWNER_EMAIL_ADDRESS;
			}
		}
		return null; // No valid email specified
	}


	/**
	* Returns immediately with whether this is operating in test mode
	* References defined variable MODULE_PAYMENT_ADYEN_MODE which
	* should be configured in the administration panel.
	* @return boolean Test Mode check
	*/
	function isTestMode() {
		if(MODULE_PAYMENT_ADYEN_MODE == "Live") {
			return false;
		}
	return true; // Default to running test mode
	}

	function isOnePagePayment() {
		 if(MODULE_PAYMENT_ADYEN_ONEPAGE == "Single Page") {
		 	return true;
		 }
		 return false;
	}

	/**
	 * Sends an email alert to the Admins email address
	 * @param String $error Verbose Error
	 */
	function sendNotifyEmail($error, &$responseArray) {
		global $order;
		$notifyAddress = $this->getNotifyEmailAddress();
		if (!empty($notifyAddress)) {
			$subject = $error;
			$message = 'Notication:' . "\n\n";
			$message .= $error . "\n\n";
			$message .= 'Payment Details' . "\n\n";
			$message .= "Adyen Result:\n";
			foreach($responseArray as $key => $value) $message .= $key . '=' . $value . "\n";
			$message .= "\n\nORDER CUSTOMER:\n";
			foreach($order->customer as $key => $value) $message .= $key . '='. $value . "\n";
			$message .= "\n\nSERVER VARS:\n";
			foreach($_SERVER as $key => $value) $message .= $key . '='. $value . "\n";
			tep_mail('', $notifyAddress, $subject, $message, STORE_OWNER, STORE_OWNER_EMAIL_ADDRESS, '');
		}
	}
}

function tep_cfg_pull_down_adyen_language($language_id, $configuration_key = NULL) {
  $name = isset($configuration_key) ? 'configuration[' . $configuration_key . ']' : 'configuration_value';

  /* languages supported by Adyen */
  $languages = array(
	'en_GB' => 'English',
	'nl' => 'Dutch',
	'de' => 'German'
  );

  $languages_array = array();

  foreach($languages as $id => $text) {
	$languages_array[] = array('id' => $id, 'text' => $text);
  }

  return tep_draw_pull_down_menu($name, $languages_array, $language_id);
}


function tep_cfg_pull_down_adyen_country($country_id, $configuration_key = NULL) {
  $name = isset($configuration_key) ? 'configuration[' . $configuration_key . ']' : 'configuration_value';

  /* countries with specific payment methods */
  $countries = array(
  	'--' => 'Determine Country by IP',
	'NL' => 'Netherlands',
	'DE' => 'Germany',
  );

  $countries_array = array();

  foreach($countries as $id => $text) {
	$countries_array[] = array('id' => $id, 'text' => $text);
  }

  return tep_draw_pull_down_menu($name, $countries_array, $country_id);
}


?>
